@@ -85,12 +85,11 @@ class Address(object):
     def __len__(self):
         return len(self.tokens)
 
-    def flat(self, slice_arg=None, *other_slice_args):
-        return u''.join(
-            u''.join(token) for token in self.tokens[
-                slice(slice_arg, *other_slice_args)
-            ]
-        )
+    def flat(self, sarg=None, *sargs):
+        return u''.join(u''.join(token) for token in self.tokens[slice(sarg, *sargs)])
+
+    def pick_to_flat(self, *idxs):
+        return u''.join(u''.join(self.tokens[idx]) for idx in idxs)
 
     def __repr__(self):
         return 'Address(%r)' % self.flat()
@@ -153,36 +152,26 @@ class Rule(Address):
 
     def match(self, addr):
 
-        # the part reserves for rule tokens
-        my_end_pos = len(self.tokens)
-        my_end_pos -= bool(self.rule_tokens) and u'全' not in self.rule_tokens
-        my_end_pos -= u'至' in self.rule_tokens
+        # except tokens reserved for rule token
 
-        his_end_pos = len(addr.tokens)
-        his_end_pos -= bool(self.rule_tokens)
+        my_last_pos = len(self.tokens)-1
+        my_last_pos -= bool(self.rule_tokens) and u'全' not in self.rule_tokens
+        my_last_pos -= u'至' in self.rule_tokens
 
-        my_tokens_to_match = self.tokens[:my_end_pos]
-        if my_tokens_to_match:
+        # tokens must be matched exactly
 
-            # the addr's tokens whose unit bigger than rule's are ignorable
-            start_unit = my_tokens_to_match[0][Address.UNIT]
-            his_start_pos = 0
-            for his_token in addr.tokens:
-                if his_token[Address.UNIT] != start_unit:
-                    his_start_pos += 1
-                else:
-                    break
+        if my_last_pos >= len(addr.tokens):
+            return False
 
-            his_tokens_to_match = addr.tokens[his_start_pos:his_end_pos]
-            if len(my_tokens_to_match) != len(his_tokens_to_match):
+        i = my_last_pos
+        while i >= 0:
+            if self.tokens[i] != addr.tokens[i]:
                 return False
-
-            for my_token, his_token in zip(my_tokens_to_match, his_tokens_to_match):
-                if my_token != his_token:
-                    return False
+            i -= 1
 
         # check the rule tokens
-        his_no_pair     = addr.parse(-1)
+
+        his_no_pair     = addr.parse(my_last_pos+1)
         my_no_pair      = self.parse(-1)
         my_asst_no_pair = self.parse(-2)
         for rt in self.rule_tokens:
@@ -287,6 +276,7 @@ class Directory(object):
         addr = Address(head_addr_str)
 
         # (a, b, c)
+
         self.put_precise(
             addr.flat(),
             head_addr_str+tail_rule_str,
@@ -294,15 +284,17 @@ class Directory(object):
         )
 
         # (a, b, c) -> (a,); (a, b); (a, b, c); (b,); (b, c); (c,)
+
         len_tokens = len(addr)
         for f in range(len_tokens):
             for l in range(f, len_tokens):
-                # only (a, b, c) -> (a, b, c); (a, c)
-                for s in range(1, 2+(f == 0 and l == 2)):
-                    self.put_gradual(
-                        addr.flat(f, l+1, s),
-                        zipcode
-                    )
+                self.put_gradual(
+                    addr.flat(f, l+1),
+                    zipcode
+                )
+
+        if len_tokens >= 3:
+            self.put_gradual(addr.pick_to_flat(0, 2), zipcode)
 
     def within_a_transaction(method):
 
